function varargout=GeneralizedNewton(f,fp,x0,varargin);
% GeneralizedNewton solves a nonlinear system of equations
%   x=GeneralizedNewton(f,fp,x0); solves the nonlinear 
%   system of equations f(x)=0 using Newtons methods with 
%   x real or complex, starting with the initial guess x0 
%   up to a tolerance tol, doing at most maxit iterations. 
%   The analytical derivative of f is given by the 
%   parameter fp. The initial guess x0 is a scalar.
%   Returns last point of the generalized Newton sequence
%     x(n+1)=x(n)-a*f(x(n))/fp(x(n))
%   (a is one by default)
%   If this sequence converge then the return value x is an
%   approximation of a root of function f.
%
% x=GeneralizedNewton(f,fp,x0,Name,Value);  
%   uses Name,Value pairs to specify the tolerence with 'tol' 
%   as Name (default Value is 1e-8), the maximum number of 
%   iterations with 'itermax' as Name (default Value is 200) 
%   and the complex parameter a with 'a' as Name 
%   corresponding to the generalization of Newton's iteration 
%   (default 1).
%
% [x,S]=GeneralizedNewton(...); 
%   returns a struct S which contains some informations ...
%     
  p = inputParser;
  p.addParamValue('tol', 1e-8, @isscalar );
  p.addParamValue('itermax', 200, @isscalar );
  p.addParamValue('a', 1, @isscalar );
  p.parse(varargin{:});
  a=p.Results.a;
  tol=p.Results.tol;
  itermax=p.Results.itermax;
  k=1;err=tol+1;alpha=[];
  x(k)=x0;
  bconv=false; % logical true if convergence
  while ( (~bconv) & (k<=itermax) )
    dx=a*f(x(k))/fp(x(k));
    x(k+1)=x(k)-dx; 
    bconv = (abs(dx)<=tol);
    k=k+1;
  end
  if nargout>=1, varargout{1}=x(end);end
  if nargout==2
    varargout{2}=struct('iter',k,'x',x,'converge',bconv, ...
                        'itermax',itermax,'tol',tol,'a',a);
  end
end