classdef mThrust < handle
% mThrust interface to the CUDA Thrust template library
%    The class mThrust makes it possible to create and use Thrust vectors 
%    stored in GPU memory (device_vector) or in CPU memory (host_vector).
%
% mThrust Properties:
%    handle    - handle to Thrust vector. 
%    container - where the Thrust vector is stored 
%                ('device' for GPU or 'host' for CPU) 
%    type      - type of the Thrust vector. Could be :
%                'double', 'float', 'int32', 'unsigned' or 'logical'.
%    dim       - dimension of the Thrust vector.
%    complex   - logical, true for complex vector or false otherwise.
%              
% mThrust Methods:
%   mThrust - Constructor (help mThrust.mThrust)
%   delete  - Destructor
  properties (SetAccess = private)
    handle    = []; % handle to Thrust vector 
    complex   = false;
    type      = []; % type of Thrust vector : 'double', 'float', 'int',  'unsigned' or 'logical'
    %typeID    = [];
    container = []; % 'host', 'device'   ( 'omp', 'tbb'?)
    %containerID =[];
    dim       = 0; % dimension of the thrust vector
  end

%    ? try < matlab.mixin.Copyable or < handle  
  
  methods
    function obj=mThrust(varargin)
% dx = mThrust( x , container ) 
%   Copies the numeric data x to Thrust vector of
%   same type. If container is 'device', the Thrust vector is on the GPU memory.
%   If container is 'host', the Thrust vector is on the CPU memory.
%   The MATLAB data x must be numeric (for example: single, double, int32 or 
%   uint32) or logical, and the GPU device must have sufficient free memory to 
%   store the data.
%
% dy = mThrust( dx ) 
%   Copies the mThrust vector dx on a new mThrust vector dy.
%
% dx = mThrust( dim,container,type)
%   Creates a mThrust vector dx of dimension 
%   dim on the GPU memory if container is 'device' or on CPU memory if 
%   container is 'host'. The type of the mThrust vector elements is given by
%   type parameter which can be 'double', 'float', 'int',  'unsigned' or 
%   'logical'
%
% dx = mThrust( dim,container,type,complex) 
%   Creates a mThrust vector dx of dimension 
%   dim on the GPU memory if container is 'device' or on CPU memory if 
%   container is 'host'. If the parameter complex is True, the mThrust vector is 
%   complex and the type of it's elements is given by type parameter which can be 
%   'double', 'float', 'int' or 'unsigned'.
%   With the parameter complex to False, the constructor is the same as the previous
%   one : dx = mThrust( dim,container,type)
%     
%   Use GATHER to copy mThrust vector back from the GPU to the MATLAB workspace.
%   
%   Example:
%     x = rand( 10,1, 'single' );
%     dx=mThrust(x,'device')
%     isequal( gather( dx ), x )  % returns true
%     dx.type                     % returns 'float'
%     dy = dx .* dx               % use "times" method defined for mThrust objects
%   
%   See also GATHER
%   
      switch nargin
        case 0
          % empty object
        case 1
          objin=varargin{1};
          assert(Thrust.ismThrust(objin))
          obj.type=objin.type;
          obj.dim=objin.dim;
          obj.container=objin.container;
          obj.complex=objin.complex;
          obj.handle=mex_mThrust_create(Thrust.toCtype(obj.dim,'int'),obj.container,obj.type,obj.complex);
          mex_mThrust_copy(objin,obj,Thrust.toCtype(0,'int'),Thrust.toCtype(obj.dim,'int'),Thrust.toCtype(0,'int'));
        case 2
          A=varargin{1};
          assert(isnumeric(A) | islogical(A),'first input value must be a numeric or logical vector');
          assert(size(A,1)==1 | size(A,2)==1,'first input value must be a Matlab vector');
          assert(ismember(varargin{2},Thrust.containers()),'2nd parameter must be a valid container');
          obj.type=Thrust.getCType(class(A));
          obj.dim=length(A);
          obj.container=varargin{2};
          obj.complex=~isreal(A);
          obj.handle=mex_mThrust_create(Thrust.toCtype(obj.dim,'int'),obj.container,obj.type,~isreal(A));
          if isreal(A)
            mex_mThrust_Matlab2Thrust(A,obj);
          else
            mex_mThrust_Matlab2ThrustCplx(real(A),imag(A),obj);
          end
        case {3,4}
          assert(ismember(varargin{2},Thrust.containers()));
          assert(ismember(varargin{3},Thrust.types()));
          if nargin==4, assert(islogical(varargin{4})); end
          obj.dim=varargin{1};
          obj.container=varargin{2};
          obj.type=varargin{3};
          obj.complex=false;
          if nargin==4
            if varargin{4}, obj.complex=true;end 
          end
          obj.handle=mex_mThrust_create(Thrust.toCtype(obj.dim,'int'),obj.container,obj.type,obj.complex);
        otherwise
          error('...');
      end
    end
    	
    % http://savannah.gnu.org/bugs/?46497
    %  with octave don't work
    function delete(obj) 
    %  Destructor 
    global tdebug
    tstart=tic();
      if obj.dim>0
        mex_mThrust_delete(obj);
        cuda.DeviceSynchronize();
      end
      tdebug=tdebug+toc(tstart);
    end
    
    
  end
  
  methods(Access = protected)
    function objcp=copyElement(obj)
      disp('in copyElement')
      objcp=mThrust(obj);
    end
  end
end