function varargout=plus(obj1,obj2,varargin)
% +  mThrust vectors addition .
%    Z = X + Y or Z=plus(X,Y) adds mThrust vectors X and Y.
%      - compatible mThrust vectors (same sizes, same types or one is logical)
%      - X a mThrust vector and Y a scalar 
%      - X a scalar and Y a mThrust vector
%    The scalar values are converted to the type of the mThrust vector.
%
%    Optional key/value pairs :
%      - 'output' key, the associated value must be a compatible mThrust 
%        vector. (experimental)
%
%    Example
%       A=Thrust.rand(1000);
%       B=Thrust.rand(1000);
%       C1=A+2;
%       C2=2+A;
%       C3=A+B;
%       C4=A+(B>0.5)
%       C5=plus(A,B);
%       plus(A,B,'output',A)  % experimental
%
% Francois Cuvelier 2013-2016
  p = inputParser;
  p.addParamValue('output', [], @(x) ( isempty(x) | Thrust.ismThrust(x) ) );
  p.parse(varargin{:});
  output=p.Results.output;
  iscomplex=(~isreal(obj1) || ~isreal(obj2));
  if ( Thrust.ismThrust(obj1) && Thrust.ismThrust(obj2) ) % mThrust<type>  <-  mThrust<type> + mThrust<type>
    assert(obj1.dim==obj2.dim,'Dimension trouble');
    assert(strcmp(obj1.container,obj2.container),'container trouble');
    if (strcmp(obj2.type,'logical'))
      if isempty(output)
        obj=mThrust(obj1.dim,obj1.container,obj1.type,iscomplex);     
      else 
        assert( iscompatible(obj1,output) );
        obj=output;
      end
      mex_mThrust_pluslogical(obj1,obj2,obj);
    elseif (strcmp(obj1.type,'logical'))
      if isempty(output)
        obj=mThrust(obj2.dim,obj2.container,obj2.type,iscomplex);
      else
        assert( iscompatible(obj2,output) );
        obj=output;
      end
      mex_mThrust_pluslogical(obj2,obj1,obj);
    elseif (strcmp(obj1.type,obj2.type))
      if isempty(output)
        obj=mThrust(obj1.dim,obj1.container,obj1.type,iscomplex);
      else
        assert( iscompatible(obj1,output,'complex',false) );
        assert( output.complex == iscomplex )
        obj=output;
      end
      mex_mThrust_plus(obj1,obj2,obj);
    end
  elseif ( (Thrust.ismThrust(obj1)) && ( isnumeric(obj2) ) ) % mThrust<type>  <-  mThrust<type> + scalar<type>
    assert(length(obj2)==1,'dimension failed');
    A=Thrust.toCtype(obj2,obj1.type);
    if isempty(output)
      obj=mThrust(obj1.dim,obj1.container,obj1.type,iscomplex);
    else
      assert( iscompatible(obj1,output,'complex',false) );
      assert( output.complex == iscomplex )
      obj=output;
    end
    mex_mThrust_pluscst(obj1,A,obj);
  elseif ( (Thrust.ismThrust(obj2)) && ( isnumeric(obj1) ) ) % mThrust<type>  <-  scalar<type> + mThrust<type>
    assert(length(obj1)==1,'dimension failed');
    A=Thrust.toCtype(obj1,obj2.type);
    if isempty(output)
      obj=mThrust(obj2.dim,obj2.container,obj2.type,iscomplex);
    else
      assert( iscompatible(obj2,output,'complex',false) );
      assert( output.complex == iscomplex )
      obj=output;
    end
    mex_mThrust_pluscst(obj2,A,obj);
  else
    error('incompatible type')
  end
  if isempty(output)
    varargout{1}=obj;
  end
end