/* class ObjectHandle: a handle class for mex-functions
 *
 * Notes:
 * 1. If passing an object to the first (pointer) constructor, be sure that
 * the object was allocated on the heap as the ObjectHandle takes ownership
 * of the objects and will eventually delete it. Be sure to not delete any aliases
 * to the passed object.
 * 2. The second (reference) constructor permits static objects to be passed via 
 * ObjectHandles by ensuring the ObjectHandle does not take ownership of
 * the object. 
 *
 * Tim Bailey 2004. Adapted from a design by Mike Stevens.
 */

#ifndef OBJECT_HANDLE_T_H_
#define OBJECT_HANDLE_T_H_

#include <mex.h>
#include <typeinfo>
#include <list>
#include <string.h>

#include "textcolors.h"

template <typename T>
class ObjectHandle {
public:
	// take ownership, and the client is responsible for deleting it.
	ObjectHandle(T& obj) : type(&typeid(T)), t(&obj) { 
		signature= this; 
	} 

	~ObjectHandle() { 
		signature= 0; // destroy signature
	} 
	
	// Convert ObjectHandle<T> to a mxArray handle (to pass back from mex-function).
	mxArray* to_mex_handle(); 

	// Convert mxArray (passed to mex-function) to an ObjectHandle<T>.
	static ObjectHandle* from_mex_handle( const mxArray* ma );
	static bool is_mex_handle( const mxArray* ma );
	static void print_mex_handle( const mxArray* ma );

	// Get the actual object contained by handle
	T& get_object() const { return *t; }

private:
	ObjectHandle* signature; // use 'this' as a unique object signature 
	const std::type_info* type; // type checking information
	T *t; // object pointer

};

// --------------------------------------------------------- 
// ---------- Implementation of member functions ----------- 
// --------------------------------------------------------- 

// Import a handle from MatLab as a mxArray of UINT32. Check that
// it is actually a pointer to an ObjectHandle<T>.
template <typename T>
ObjectHandle<T>* ObjectHandle<T>::from_mex_handle(const mxArray* handle) 
{
  if (mxGetClassID(handle) != mxDOUBLE_CLASS 
    || mxIsComplex(handle) || mxGetM(handle)!=1 || mxGetN(handle)!=1)
    mexErrMsgTxt("Parameter is not an ObjectHandle type.");
  
  // We *assume* we can store ObjectHandle<T> pointer in the mxUINT32 of handle
    ObjectHandle* obj = *reinterpret_cast<ObjectHandle<T>**>(mxGetPr(handle));
    
    if (!obj) // gross check to see we don't have an invalid pointer
      mexErrMsgTxt("Parameter is NULL. It does not represent an ObjectHandle object.");
    // TODO: change this for max-min check for pointer values
    if (obj->signature != obj) // check memory has correct signature
      mexErrMsgTxt("Parameter does not represent an ObjectHandle object.");
    //mexPrintf("obj->type->name() : %s, typeid(T).name()=%s\n",obj->type->name(),typeid(T).name());
    if ( strcmp(obj->type->name(),typeid(T).name()) != 0 ) 
      mexErrMsgTxt("Given ObjectHandle does not represent the correct type.");
    
    return obj;
}

template <typename T> bool ObjectHandle<T>::is_mex_handle( const mxArray* handle ){
  if (mxGetClassID(handle) != mxDOUBLE_CLASS 
      || mxIsComplex(handle) || mxGetM(handle)!=1 || mxGetN(handle)!=1)
    return false;
 
  // We *assume* we can store ObjectHandle<T> pointer in the mxUINT32 of handle
  ObjectHandle* obj = *reinterpret_cast<ObjectHandle<T>**>(mxGetPr(handle));
    
  if (!obj) // gross check to see we don't have an invalid pointer
    return false;

  if (obj->signature != obj) // check memory has correct signature
    return false;
    //mexPrintf("obj->type->name() : %s, typeid(T).name()=%s\n",obj->type->name(),typeid(T).name());
  if ( strcmp(obj->type->name(),typeid(T).name()) != 0 ) 
    return false;
  return true;
}

template <typename T> void ObjectHandle<T>::print_mex_handle( const mxArray* handle ){
  ObjectHandle* obj = from_mex_handle(handle);
  //mexPrintf("%sObjectHandle print :\n  type->name() : %s%s\n",TEXTCOLOR_GREEN, obj->type->name(),TEXTCOLOR_END);
  mexPrintf("ObjectHandle print :  type->name() : %s [%s:%d]\n", obj->type->name(),__FILE__,__LINE__);
  //std::cout << "ObjectHandle print :\n  type->name() : " << obj->type->name() << std::endl;
}

// Create a numeric array as handle for an ObjectHandle.
// We ASSUME we can store object pointer in the mxUINT32 element of mxArray.
template <typename T>
mxArray* ObjectHandle<T>::to_mex_handle() 
{
  mxArray* handle  = mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
  *reinterpret_cast<ObjectHandle<T>**>(mxGetPr(handle)) = this;
  return handle;
}

// --------------------------------------------------------- 
// ------------------ Helper functions ---------------------
// --------------------------------------------------------- 
// These functions remove the need to deal with ObjectHandle<T>
// class directly for most common operations.

//template <typename T> static void delete_handle (ObjectHandle<T>* obj);

template <typename T> mxArray *create_handle(T* t)
// Create mex handle to object t (where t is heap allocated). 
// Client no longer owns t, and so must not delete it.
{
	ObjectHandle<T>* handle= new ObjectHandle<T>(t);
	return handle->to_mex_handle();
}

template <typename T> T& get_object(const mxArray *mxh)
// Obtain object represented by handle.
{
	ObjectHandle<T>* handle= ObjectHandle<T>::from_mex_handle(mxh);
	return handle->get_object();
}

template <typename T> void destroy_object(const mxArray *mxh)
// If deleting object, rather than leaving it to garbage collection,
// must delete it via the handle; do not delete T* directly.
{
	ObjectHandle<T>* handle= ObjectHandle<T>::from_mex_handle(mxh);
	delete handle;
}


#endif
