#include <string>
//#include <sstream>
#include <cuda.h>
//#include <helper_cuda.h>
#include "textcolors.h"
#include "mex.h"
//#define VERBOSE
#include <stdio.h>

// From helper_cuda.h
// Beginning of GPU Architecture definitions
int _ConvertSMVer2Cores(int major, int minor)
{
  // Defines for GPU Architecture types (using the SM version to determine the # of cores per SM
  typedef struct
  {
    int SM; // 0xMm (hexidecimal notation), M = SM Major version, and m = SM minor version
    int Cores;
  } sSMtoCores;
  
  sSMtoCores nGpuArchCoresPerSM[] =
  {
    { 0x20, 32 }, // Fermi Generation (SM 2.0) GF100 class
    { 0x21, 48 }, // Fermi Generation (SM 2.1) GF10x class
    { 0x30, 192}, // Kepler Generation (SM 3.0) GK10x class
    { 0x32, 192}, // Kepler Generation (SM 3.2) GK10x class
    { 0x35, 192}, // Kepler Generation (SM 3.5) GK11x class
    { 0x37, 192}, // Kepler Generation (SM 3.7) GK21x class
    { 0x50, 128}, // Maxwell Generation (SM 5.0) GM10x class
    { 0x52, 128}, // Maxwell Generation (SM 5.2) GM20x class
    {   -1, -1 }
  };
  
  int index = 0;
  
  while (nGpuArchCoresPerSM[index].SM != -1)
  {
    if (nGpuArchCoresPerSM[index].SM == ((major << 4) + minor))
    {
      return nGpuArchCoresPerSM[index].Cores;
    }
    
    index++;
  }
  
  // If we don't find the values, we default use the previous one to run properly
  mexPrintf("MapSMtoCores for SM %d.%d is undefined.  Default to use %d Cores/SM\n", major, minor, nGpuArchCoresPerSM[index-1].Cores);
  return nGpuArchCoresPerSM[index-1].Cores;
}
// end of GPU Architecture definitions

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] ){
  int devCount;
  cudaGetDeviceCount(&devCount);
  if (devCount == 0)
  {
    mexPrintf("There are no available device(s) that support CUDA\n");
    plhs[0] = mxCreateDoubleMatrix( 0, 0, mxREAL );
    return;
  }
  int DeviceNumber=-1;
  if (nrhs==0){
    cudaGetDevice(&DeviceNumber);
  }else{
    DeviceNumber =(int) *( (double *) mxGetPr(prhs[0]));
    if ((DeviceNumber<0) || (DeviceNumber>=devCount)){
      mexPrintf("CUDA device %d not available\n",DeviceNumber);
      plhs[0] = mxCreateDoubleMatrix( 0, 0, mxREAL );
      return;
    }
  }
  mwSize ndim = 2;
  const mwSize dims[2] = {1, 1};
  int number_of_fields=15;   
  const char *field_names[] = {"Name", "Index","ComputeCapability","RuntimeVersion","DriverVersion",
    "TotalMemory","AvailableMemory","MultiprocessorCount","CoresByMP","CudaCores","GPUMaxClockRateMhz",
    "MaxThreadsPerBlock","SharedMemPerBlock","MaxThreadBlockSize","MaxGridSize" };
  mxArray *struct_array_ptr;
  struct_array_ptr = mxCreateStructArray(ndim, dims, 
                                         number_of_fields, field_names);
  plhs[0]=struct_array_ptr;
  // <- Initialize output structure
  // Get device properties
    cudaDeviceProp deviceProp;
    cudaGetDeviceProperties(&deviceProp, DeviceNumber);
    
    size_t free_byte, total_byte ;
    cudaMemGetInfo( &free_byte, &total_byte ) ;
    
    mxSetField(struct_array_ptr, 0, "Name", mxCreateString(deviceProp.name));
    int driverVersion = 0, runtimeVersion = 0;
    cudaDriverGetVersion(&driverVersion);
    cudaRuntimeGetVersion(&runtimeVersion);
    mxArray  *DriverVersion = mxCreateDoubleMatrix(1, 1, mxREAL);
    mxArray  *RuntimeVersion = mxCreateDoubleMatrix(1, 1, mxREAL);
    mxSetField(struct_array_ptr, 0, "Index", mxCreateDoubleScalar(DeviceNumber));
    mxSetField(struct_array_ptr, 0, "DriverVersion", mxCreateDoubleScalar(driverVersion/1000+(driverVersion%100)/100.));
    mxSetField(struct_array_ptr, 0, "RuntimeVersion", mxCreateDoubleScalar(runtimeVersion/1000+(runtimeVersion%100)/100.));
    mxSetField(struct_array_ptr, 0, "TotalMemory",mxCreateDoubleScalar((double)deviceProp.totalGlobalMem));
    mxSetField(struct_array_ptr, 0, "AvailableMemory",mxCreateDoubleScalar((double)(free_byte)));
    
    mxSetField(struct_array_ptr, 0, "MultiprocessorCount",mxCreateDoubleScalar((double)deviceProp.multiProcessorCount));
    mxSetField(struct_array_ptr, 0, "CoresByMP",mxCreateDoubleScalar((double)_ConvertSMVer2Cores(deviceProp.major, deviceProp.minor)));
    mxSetField(struct_array_ptr, 0, "CudaCores",mxCreateDoubleScalar((double)(_ConvertSMVer2Cores(deviceProp.major, deviceProp.minor)* deviceProp.multiProcessorCount)));
    mxSetField(struct_array_ptr, 0, "GPUMaxClockRateMhz",mxCreateDoubleScalar((double)(deviceProp.clockRate * 1e-3f)));
    
    mxSetField(struct_array_ptr, 0, "MaxThreadsPerBlock",mxCreateDoubleScalar((double)(deviceProp.maxThreadsPerBlock)));
    mxSetField(struct_array_ptr, 0, "SharedMemPerBlock",mxCreateDoubleScalar((double)(deviceProp.sharedMemPerBlock)));
    mxArray  *MaxThreadBlockSize = mxCreateDoubleMatrix(1, 3, mxREAL);
    double *pr = mxGetPr(MaxThreadBlockSize);
    for (int i=0;i<3;i++) pr[i]=deviceProp.maxThreadsDim[i];
    mxSetField(struct_array_ptr, 0, "MaxThreadBlockSize", MaxThreadBlockSize);
    mxSetField(struct_array_ptr, 0, "MaxGridSize", mxCreateDoubleMatrix(1, 3, mxREAL));
    pr=mxGetPr(mxGetField(struct_array_ptr, 0, "MaxGridSize"));
    for (int i=0;i<3;i++) pr[i]=deviceProp.maxGridSize[i];
    
    char cTemp[10];
    #if defined(WIN32) || defined(_WIN32) || defined(WIN64) || defined(_WIN64)
    sprintf_s(cTemp, 10, "%d.%d", deviceProp.major, deviceProp.minor);
    #else
    sprintf(cTemp, "%d.%d", deviceProp.major, deviceProp.minor);
    #endif    
    mxSetField(struct_array_ptr, 0, "ComputeCapability", mxCreateString( (const char *)cTemp));
}