/*! \file mex_mThrust_plus.cu
 *  \brief MEX-file : delete a C++ \p gTArray object (GPU).
 * 
 *  The gateway Cuda/C++ function \p mex_gTArrayDestroy destroy a C++ \p gTArray object 
 *  pointed by a handle.
 * 
 *  &copy;Matlab syntax <tt><B>mex_gTArrayDelete(h,type)</B></tt> where
 *    - \p h  handle representing a C++ \p gTArray object.
 *    - \p type of \p gTArray object given by \p h. Must be 'int', 'unsigned', 'float' or 'double'.
 * 
 *  The following &copy;Matlab code snippet demonstrates how to use this gateway function \p mex_gTArrayDelete 
 *  \code
 *  >> hA=mex_gTArrayCreate(int32(500),'double');
 *  ...
 *  >> mex_gTArrayDelete(hA,'double');
 *  \endcode
 * 
 * \sa  mex_gTArrayCreate
 */
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"

#define MESS "mThrustToolbox:Matlab2Thrust"

template <typename T,typename Vector> 
void mx_mThrust_op(const mxArray *mexhandle_x,
                   const mxArray *mx_mThrust
){
  
  T *px = (T *)mxGetPr(mexhandle_x);
  //mxGetNumberOfElements(px);
  Vector &gy = get_object< Vector >(mx_mThrust);
  //mxGetM(mexhandle_x));mxGetN(mexhandle_x));
  mxAssert( ((int)mxGetNumberOfElements(mexhandle_x)) == gy.size() ,"dimension trouble");
  // 
  //assert( strcmp( mxArrayToString(mxGetProperty(mexhandle_x,0,"type")), mxArrayToString(mxGetProperty(mexhandle_y,0,"type")) ) == 0);
  //assert( strcmp( mxArrayToString(mxGetProperty(mexhandle_x,0,"type")), mxArrayToString(mxGetProperty(mexhandle_z,0,"type")) ) == 0);
  thrust::copy(px,px+gy.size(), gy.begin());
}

template <typename T,typename Vector> 
void mx_mThrust_trans(const mxArray *prhs[]){
  Vector &gx = get_object< Vector >(mxGetProperty(prhs[1],0,"handle"));
  mxAssert( (int)mxGetNumberOfElements(prhs[0]) == gx.size() ,"dimension trouble");
  
  T *px = (T *)mxGetPr(prhs[0]);
  thrust::copy(px,px+gx.size(), gx.begin());
}

template  <typename T>
void mx_mThrust_trans_type(const mxArray *prhs[]){
  char *container= mxArrayToString(mxGetProperty(prhs[1],0,"container"));
  
  if (strcmp(container, "device")==0){
    mx_mThrust_trans<T,thrust::device_vector<T> >(prhs); 
  }else if (strcmp(container, "host")==0){
    mx_mThrust_trans<T,thrust::host_vector<T > >(prhs); 
  }else{
    mxFree(container);
    mexErrMsgIdAndTxt(MESS,"Unknow container."); 
  }
  mxFree(container);
}

// prhs[0] : Matlab array (not complex)
// prhs[1] : input/output mThrust object (not complex)
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 2) 
    mexErrMsgTxt("Bad input.");
  mxAssert(mxIsClass(prhs[1],"mThrust"),"2nd parameter must be a mThrust object");
  char *type =mxArrayToString(mxGetProperty(prhs[1],0,"type"));
  if (strcmp(type, "int")==0){
    mxAssert( mxGetClassID(prhs[0])==mxINT32_CLASS,"type trouble, expected int");
    mx_mThrust_trans_type<int>(prhs);
  }else if (strcmp(type, "unsigned")==0){
    mxAssert( mxGetClassID(prhs[0])==mxUINT32_CLASS ,"type trouble, expected unsigned");
    mx_mThrust_trans_type<unsigned>(prhs);
  }else if (strcmp(type, "float")==0){
    mxAssert( mxGetClassID(prhs[0])==mxSINGLE_CLASS ,"type trouble, expected float");
    mx_mThrust_trans_type<float>(prhs);
  }else if (strcmp(type, "double")==0){
    mxAssert( mxGetClassID(prhs[0])==mxDOUBLE_CLASS ,"type trouble, expected double");
    mx_mThrust_trans_type<double>(prhs);
  }else if (strcmp(type, "logical")==0){
    mxAssert( mxGetClassID(prhs[0])==mxLOGICAL_CLASS,"type trouble, expected logical");
    mx_mThrust_trans_type<logical>(prhs);
  }else{
    mxFree(type);
    mexErrMsgIdAndTxt(MESS,"type not implemented.");
  }
  mxFree(type);
  MEXPRINTF("ending");
}

// void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
// {
//   MEXPRINTF("starting");
//   if (nlhs !=0 || nrhs != 4) 
//     mexErrMsgTxt("Bad input.");
//   //assert(mxIsClass(prhs[0],"mThrust"));
//   char *type =mxArrayToString(prhs[3]);
//   char *container =mxArrayToString(prhs[2]);
//   //const mxArray *mx_mThrust=prhs[1];
//   
//     //assert( strcmp( type , mxArrayToString(mxGetProperty(prhs[1],0,"type")) ) == 0);
//   //assert( strcmp( container , mxArrayToString(mxGetProperty(prhs[1],0,"container")) ) == 0);
//   
//   if (strcmp(type, "int")==0){
//     if (strcmp(container, "host")==0){
//       mx_mThrust_op<int,thrust::host_vector<int> >(prhs[0],prhs[1]);
//     }else if (strcmp(container, "device")==0){
//       mx_mThrust_op<int,thrust::device_vector<int> >(prhs[0],prhs[1]);
//     }else
//       mexErrMsgIdAndTxt("mThrustToolbox:Matlab2Thrust","Unknow container.");
//   }else if (strcmp(type, "unsigned")==0){
//     if (strcmp(container, "host")==0){
//       mx_mThrust_op<unsigned,thrust::host_vector<unsigned> >(prhs[0],prhs[1]);
//     }else if (strcmp(container, "device")==0){
//       mx_mThrust_op<unsigned,thrust::device_vector<unsigned> >(prhs[0],prhs[1]);
//     }else
//       mexErrMsgIdAndTxt("mThrustToolbox:Matlab2Thrust","Unknow container.");
//   }else if (strcmp(type, "float")==0){
//     if (strcmp(container, "host")==0){
//       mx_mThrust_op<float,thrust::host_vector<float> >(prhs[0],prhs[1]);
//     }else if (strcmp(container, "device")==0){
//       mx_mThrust_op<float,thrust::device_vector<float> >(prhs[0],prhs[1]);
//     }else
//       mexErrMsgIdAndTxt("mThrustToolbox:Matlab2Thrust","Unknow container.");
//   }else if (strcmp(type, "double")==0){
//     if (strcmp(container, "host")==0){
//       mx_mThrust_op<double,thrust::host_vector<double> >(prhs[0],prhs[1]);
//     }else if (strcmp(container, "device")==0){
//       mx_mThrust_op<double,thrust::device_vector<double> >(prhs[0],prhs[1]);
//     }else
//       mexErrMsgIdAndTxt("mThrustToolbox:Matlab2Thrust","Unknow container.");
//   }else  if (strcmp(type, "logical")==0){
//     if (strcmp(container, "host")==0){
//       mx_mThrust_op<logical,thrust::host_vector<logical> >(prhs[0],prhs[1]);
//     }else if (strcmp(container, "device")==0){
//       mx_mThrust_op<logical,thrust::device_vector<logical> >(prhs[0],prhs[1]);
//     }else
//       mexErrMsgIdAndTxt("mThrustToolbox:Matlab2Thrust","Unknow container.");
//   }else
//     mexErrMsgIdAndTxt("mThrustToolbox:uplus","type not implemented.");
// }