/*! \file mex_mThrust_plus.cu
 *  \brief MEX-file : delete a C++ \p gTArray object (GPU).
 * 
 *  The gateway Cuda/C++ function \p mex_gTArrayDestroy destroy a C++ \p gTArray object 
 *  pointed by a handle.
 * 
 *  &copy;Matlab syntax <tt><B>mex_gTArrayDelete(h,type)</B></tt> where
 *    - \p h  handle representing a C++ \p gTArray object.
 *    - \p type of \p gTArray object given by \p h. Must be 'int', 'unsigned', 'float' or 'double'.
 * 
 *  The following &copy;Matlab code snippet demonstrates how to use this gateway function \p mex_gTArrayDelete 
 *  \code
 *  >> hA=mex_gTArrayCreate(int32(500),'double');
 *  ...
 *  >> mex_gTArrayDelete(hA,'double');
 *  \endcode
 * 
 * \sa  mex_gTArrayCreate
 */
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"
#include <thrust/transform.h>
#include <thrust/copy.h>


#define MESS "mThrustToolbox:Matlab2ThrustCplx"

template <typename T> 
struct trans_binary_functor : public thrust::binary_function<T,T,thrust::complex<T> >
{
  __host__ __device__
  thrust::complex<T> operator()(const T & x,const T & y) const { 
    return thrust::complex<T> (x,y);
  }
};

template <typename T,typename Vector,typename VectorReal> 
void mx_mThrust_cplxtrans(const mxArray *prhs[]){
  Vector &gz = get_object< Vector >(mxGetProperty(prhs[2],0,"handle"));
  mxAssert( ((int)mxGetNumberOfElements(prhs[0])) == gz.size() ,"dimension trouble");
  mxAssert( ((int)mxGetNumberOfElements(prhs[1])) == gz.size() ,"dimension trouble");
  
  T *px = (T *)mxGetPr(prhs[0]);
  T *py = (T *)mxGetPr(prhs[1]);
  int N=gz.size();
  VectorReal gx(px,px+N);
  VectorReal gy(py,py+N);
  thrust::transform(gx.begin(),gx.end(),gy.begin(), gz.begin(),trans_binary_functor<T>());
}

template  <typename T>
void mx_mThrust_cplxtrans_type(const mxArray *prhs[]){
  char *container= mxArrayToString(mxGetProperty(prhs[2],0,"container"));
  
  if (strcmp(container, "device")==0){
    mx_mThrust_cplxtrans<T, thrust::device_vector<thrust::complex<T> >, thrust::device_vector<T> >(prhs); 
  }else if (strcmp(container, "host")==0){
    mx_mThrust_cplxtrans<T, thrust::host_vector<thrust::complex<T> >,   thrust::host_vector<T> >(prhs); 
  }else{
    mxFree(container);
    mexErrMsgIdAndTxt(MESS,"Unknow container."); 
  }
  mxFree(container);
}

// prhs[0] : real part (Matlab)
// prhs[1] : imag part (Matlab)
// prhs[2] : input/output mThrust object (complex)
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 3) 
    mexErrMsgTxt("Bad input.");
  mxAssert(mxIsClass(prhs[2],"mThrust"),"3rd parameter must be a complex mThrust object");
  char *type =mxArrayToString(mxGetProperty(prhs[2],0,"type"));
  if (strcmp(type, "int")==0){
    mxAssert( (mxGetClassID(prhs[0])==mxINT32_CLASS) && (mxGetClassID(prhs[1])==mxINT32_CLASS),"type trouble, expected int");
    mx_mThrust_cplxtrans_type<int>(prhs);
  }else if (strcmp(type, "unsigned")==0){
    mxAssert( (mxGetClassID(prhs[0])==mxUINT32_CLASS) && (mxGetClassID(prhs[1])==mxUINT32_CLASS),"type trouble, expected unsigned");
    mx_mThrust_cplxtrans_type<unsigned>(prhs);
  }else if (strcmp(type, "float")==0){
    mxAssert( (mxGetClassID(prhs[0])==mxSINGLE_CLASS) && (mxGetClassID(prhs[1])==mxSINGLE_CLASS),"type trouble, expected float");
    mx_mThrust_cplxtrans_type<float>(prhs);
  }else if (strcmp(type, "double")==0){
    mxAssert( (mxGetClassID(prhs[0])==mxDOUBLE_CLASS) && (mxGetClassID(prhs[1])==mxDOUBLE_CLASS),"type trouble, expected double");
    mx_mThrust_cplxtrans_type<double>(prhs);
  }else{
    mxFree(type);
    mexErrMsgIdAndTxt(MESS,"type not implemented.");
  }
  mxFree(type);
  MEXPRINTF("ending");
}