/*! \file mex_gTArrayDelete.cu
 *  \brief MEX-file : delete a C++ \p gTArray object (GPU).
 * 
 *  The gateway Cuda/C++ function \p mex_gTArrayDestroy destroy a C++ \p gTArray object 
 *  pointed by a handle.
 * 
 *  &copy;Matlab syntax <tt><B>mex_gTArrayDelete(h,type)</B></tt> where
 *    - \p h  handle representing a C++ \p gTArray object.
 *    - \p type of \p gTArray object given by \p h. Must be 'int', 'unsigned', 'float' or 'double'.
 * 
 *  The following &copy;Matlab code snippet demonstrates how to use this gateway function \p mex_gTArrayDelete 
 *  \code
 *  >> hA=mex_gTArrayCreate(int32(500),'double');
 *  ...
 *  >> mex_gTArrayDelete(hA,'double');
 *  \endcode
 * 
 * \sa  mex_gTArrayCreate
 */
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"
#include <thrust/complex.h>

#define MESS "mThrustToolbox:debug"

void PrintMemorySize(unsigned size){
  //unsigned size=(unsigned)dim*sizeof(T);
  double conv=(double) size/1000000000.;
  if (conv >=1){
    printf("size: %.3lf Go",conv);
    return;
  }
  conv=(double) size/1000000.;
  if (conv >=1){
    printf("size: %.3lf Mo",conv);
    return;
  }
  conv=(double) size/1000.;
  if (conv >=1){
    printf("size: %.3lf Ko",conv);
    return;
  }
  printf("size: %u octets",size);
}

#define PRINT_DEBUG1(C,T,N,A,S) printf("%s  - Thrust %s_vector<%s>[%d]\n      memory address: %#lX\n                ",GPUMEMORYCOLOR,C,T,N,(MEMORY_TYPE)A),PrintMemorySize(S),printf("%s\n",TEXTCOLOR_END);

#define PRINT_DEBUG_CPLX(C,T,N,A,S) printf("%s  - Thrust %s_vector<complex<%s>>[%d]\n      memory address: %#lX\n                ",GPUMEMORYCOLOR,C,T,N,(MEMORY_TYPE)A),PrintMemorySize(S),printf("%s\n",TEXTCOLOR_END);


template <typename T,typename Vector> void  mThrust_debug_op_complex(const mxArray *prhs[],
                                                                  char *container,char *type){
  Vector &Vr = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  typedef typename Vector::value_type  TV;
  PRINT_DEBUG_CPLX(container,type,Vr.size(),&Vr,(int)(Vr.size()*sizeof(TV)));
}

template <typename T,typename Vector> void  mThrust_debug_op_real(const mxArray *prhs[],
                                                                  char *container,char *type){
  Vector &Vr = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  typedef typename Vector::value_type  TV;
  PRINT_DEBUG1(container,type,Vr.size(),&Vr,(int)(Vr.size()*sizeof(TV)));
}

template  <typename T>
void mx_mThrust_debug_type(const mxArray *prhs[],char *container,char *type,logical iscomplex){
  if (strcmp(container, "host")==0){
    if (iscomplex){
      mThrust_debug_op_complex<T,thrust::host_vector<thrust::complex<T> > >(prhs,container,type);
    }else{
      mThrust_debug_op_real<T,thrust::host_vector<T> >(prhs,container,type);
    }
  }else if (strcmp(container, "device")==0){
    if (iscomplex){
      mThrust_debug_op_complex<T,thrust::device_vector<thrust::complex<T> > >(prhs,container,type);
    }else{
      mThrust_debug_op_real<T,thrust::device_vector<T> >(prhs,container,type);
    }
  }else
    mexErrMsgIdAndTxt(MESS,"Unknow container."); 
}

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 1) 
    mexErrMsgTxt("Bad input.");
  assert(mxIsClass(prhs[0],"mThrust"));
  logical *pisComplex=(logical *) mxGetPr(mxGetProperty(prhs[0],0,"complex"));
  char *type =mxArrayToString(mxGetProperty(prhs[0],0,"type"));
  char *container =mxArrayToString(mxGetProperty(prhs[0],0,"container"));
  
  if (strcmp(type, "int")==0)
    mx_mThrust_debug_type<int>(prhs,container,type,*pisComplex);
  else if (strcmp(type, "double")==0)
    mx_mThrust_debug_type<double>(prhs,container,type,*pisComplex);
  else if (strcmp(type, "float")==0)
    mx_mThrust_debug_type<float>(prhs,container,type,*pisComplex);
  else if (strcmp(type, "unsigned")==0)
    mx_mThrust_debug_type<unsigned>(prhs,container,type,*pisComplex);
  else if (strcmp(type, "logical")==0)
    mx_mThrust_debug_type<logical>(prhs,container,type,*pisComplex);
  else{ 
    mxFree(container);mxFree(type);
    mexErrMsgIdAndTxt(MESS,"Type not implemented.");
  }
  mxFree(container);mxFree(type);
  MEXPRINTF("ending");
}