/*! \file mex_gTArrayDelete.cu
 *  \brief MEX-file : delete a C++ \p gTArray object (GPU).
 * 
 *  The gateway Cuda/C++ function \p mex_gTArrayDestroy destroy a C++ \p gTArray object 
 *  pointed by a handle.
 * 
 *  &copy;Matlab syntax <tt><B>mex_gTArrayDelete(h,type)</B></tt> where
 *    - \p h  handle representing a C++ \p gTArray object.
 *    - \p type of \p gTArray object given by \p h. Must be 'int', 'unsigned', 'float' or 'double'.
 * 
 *  The following &copy;Matlab code snippet demonstrates how to use this gateway function \p mex_gTArrayDelete 
 *  \code
 *  >> hA=mex_gTArrayCreate(int32(500),'double');
 *  ...
 *  >> mex_gTArrayDelete(hA,'double');
 *  \endcode
 * 
 * \sa  mex_gTArrayCreate
 */
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"


void PrintMemorySize(unsigned size){
  //unsigned size=(unsigned)dim*sizeof(T);
  double conv=(double) size/1000000000.;
  if (conv >=1){
    printf("size: %.3lf Go",conv);
    return;
  }
  conv=(double) size/1000000.;
  if (conv >=1){
    printf("size: %.3lf Mo",conv);
    return;
  }
  conv=(double) size/1000.;
  if (conv >=1){
    printf("size: %.3lf Ko",conv);
    return;
  }
  printf("size: %u octets",size);
}

#define PRINT_DEBUG1(C,T,N,A,S) printf("%s  - Thrust %s_vector<%s>[%d] (\n  memory address: %#lX,\n           ",GPUMEMORYCOLOR,C,T,N,(MEMORY_TYPE)A),PrintMemorySize(S),printf(")%s\n",TEXTCOLOR_END);

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 1) 
    mexErrMsgTxt("Bad input.");
  assert(mxIsClass(prhs[0],"mThrust"));
  char *type =mxArrayToString(mxGetProperty(prhs[0],0,"type"));
  char *container =mxArrayToString(mxGetProperty(prhs[0],0,"container"));
  mxArray *mxhandle=mxGetProperty(prhs[0],0,"handle");
  
  if (strcmp(container, "host")==0){  
    if (strcmp(type, "int")==0){
      thrust::host_vector<int> &pV=get_object< thrust::host_vector<int> >(mxhandle);
      int size=pV.size();
      PrintMemorySize((unsigned)(pV.size()*sizeof(int)));
      PRINT_DEBUG1(container,type,pV.size(),&pV,(int)(pV.size()*sizeof(int)));
/*#ifdef DEBUG_GPUMEMORY            
      printf("%sdebug : %s_vector<%s>[%d] (memory adress: %#lX)%s\n",MEXCOLOR,container,type,pV.size(),(MEMORY_TYPE)&pV,TEXTCOLOR_END);
#endif*/
      /*for(size_t i = 0; i < pV->size(); i++)
        std::cout << "V[" << i << "] = " << (*pV)[i] << std::endl;*/
    }else if (strcmp(type, "double")==0){
      thrust::host_vector<double> &pV=get_object< thrust::host_vector<double> >(mxhandle);    
      PRINT_DEBUG1(container,type,pV.size(),&pV,(int)(pV.size()*sizeof(double)));
    }else if (strcmp(type, "float")==0){
      thrust::host_vector<float> &pV=get_object< thrust::host_vector<float> >(mxhandle);   
      PRINT_DEBUG1(container,type,pV.size(),&pV,(int)(pV.size()*sizeof(float)));
    }else if (strcmp(type, "unsigned")==0){
      thrust::host_vector<unsigned> &pV=get_object< thrust::host_vector<unsigned> >(mxhandle);
      PRINT_DEBUG1(container,type,pV.size(),&pV,(int)(pV.size()*sizeof(unsigned)));
    }else if (strcmp(type, "logical")==0){
      thrust::host_vector<logical> &pV=get_object< thrust::host_vector<logical> >(mxhandle); 
      PRINT_DEBUG1(container,type,pV.size(),&pV,(int)(pV.size()*sizeof(logical)));
    }else
      mexErrMsgIdAndTxt("mThrustToolbox:create","Type not implemented.");
  }else if (strcmp(container, "device")==0){ 
    if (strcmp(type, "int")==0){
      thrust::device_vector<int> &pV=get_object< thrust::device_vector<int> >(mxhandle);  
      PRINT_DEBUG1(container,type,pV.size(),&pV,(int)(pV.size()*sizeof(int)));
      for(size_t i = 0; i < pV.size(); i++)
        std::cout << "V[" << i << "] = " << pV[i] << std::endl;
    }else if (strcmp(type, "double")==0){
      thrust::device_vector<double> &pV=get_object< thrust::device_vector<double> >(mxhandle); 
      PRINT_DEBUG1(container,type,pV.size(),&pV,(unsigned)(pV.size()*sizeof(double)));
    }else if (strcmp(type, "float")==0){
      thrust::device_vector<float> &pV=get_object< thrust::device_vector<float> >(mxhandle);
      PRINT_DEBUG1(container,type,pV.size(),&pV,(unsigned)(pV.size()*sizeof(float)));
    }else if (strcmp(type, "unsigned")==0){
      thrust::device_vector<unsigned> &pV=get_object< thrust::device_vector<unsigned> >(mxhandle);   
      PRINT_DEBUG1(container,type,pV.size(),&pV,(unsigned)(pV.size()*sizeof(float)));
    }else if (strcmp(type, "logical")==0){
      thrust::device_vector<logical> &pV=get_object< thrust::device_vector<logical> >(mxhandle);
      PRINT_DEBUG1(container,type,pV.size(),&pV,(unsigned)(pV.size()*sizeof(logical)));
    }else
      mexErrMsgIdAndTxt("mThrustToolbox:create","Type not implemented.");
  }else
    mexErrMsgIdAndTxt("mThrustToolbox:create","Unknow container.");
}