/*! \file mex_mThrust_set.cu
 *  \brief MEX-file : delete a C++ \p gTArray object (GPU).
 * 
 *  The gateway Cuda/C++ function \p mex_gTArrayDestroy destroy a C++ \p gTArray object 
 *  pointed by a handle.
 * 
 *  &copy;Matlab syntax <tt><B>mex_gTArrayDelete(h,type)</B></tt> where
 *    - \p h  handle representing a C++ \p gTArray object.
 *    - \p type of \p gTArray object given by \p h. Must be 'int', 'unsigned', 'float' or 'double'.
 * 
 *  The following &copy;Matlab code snippet demonstrates how to use this gateway function \p mex_gTArrayDelete 
 *  \code
 *  >> hA=mex_gTArrayCreate(int32(500),'double');
 *  ...
 *  >> mex_gTArrayDelete(hA,'double');
 *  \endcode
 * 
 * \sa  mex_gTArrayCreate
 */
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"

#define MESS "mThrustToolbox:eq"

template <typename T,typename Vector,typename VectorBool> 
void mx_mThrust_opVec(const mxArray *prhs[]){
  Vector &gx = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  Vector &gy = get_object< Vector >(mxGetProperty(prhs[1],0,"handle"));
  VectorBool &gz = get_object< VectorBool >(mxGetProperty(prhs[2],0,"handle"));
  assert((gx.size()==gy.size()) && (gx.size()==gz.size()));
  thrust::transform(gx.begin(),gx.end(), gy.begin(),gz.begin(), thrust::not_equal_to<T>());
}

template <typename T> struct op_functor : public thrust::unary_function<T,T>
{
    const T a;

    op_functor(T _a) : a(_a) {}

    __host__ __device__
        bool operator()(const T& x) const { 
            return x!=a;
        }
};

template <typename T,typename Vector,typename VectorBool> 
void mx_mThrust_opCstReal(const mxArray *prhs[]){
  Vector &gx = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  VectorBool &gz = get_object< VectorBool >(mxGetProperty(prhs[2],0,"handle"));
  assert( (gx.size()==gz.size()));
  T *py = (T *)mxGetPr(prhs[1]);
  thrust::transform(gx.begin(),gx.end(),gz.begin(), op_functor<T>(*py));
}

template <typename T,typename Vector,typename VectorBool> 
void mx_mThrust_opCstComplex(const mxArray *prhs[]){
  Vector &gx = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  VectorBool &gz = get_object< VectorBool >(mxGetProperty(prhs[2],0,"handle"));
  assert( (gx.size()==gz.size()));
  T *pr = (T *)mxGetPr(prhs[1]);
  T *pi = (T *)mxGetPi(prhs[1]);
  thrust::complex<T> c(*pr,*pi);
  thrust::transform(gx.begin(),gx.end(),gz.begin(), op_functor<thrust::complex<T> >(c));
}

// template <typename T,typename Vector,typename VectorBool> 
// void mx_mThrust_opCst(const mxArray *prhs[]){
//   
//   
//   Vector &gx = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
//   T *py = (T *)mxGetPr(prhs[1]);
//   VectorBool &gz = get_object< VectorBool >(mxGetProperty(prhs[2],0,"handle"));
//   assert( (gx.size()==gz.size()));
//   thrust::transform(gx.begin(),gx.end(),gz.begin(), op_functor<T>(*py));
// }

// template  <typename T>
// void mx_mThrust_eqVec_type( const mxArray *prhs[] ){
//   char *container[] = {
//     mxArrayToString(mxGetProperty(prhs[0],0,"container")),
//     mxArrayToString(mxGetProperty(prhs[1],0,"container")),
//     mxArrayToString(mxGetProperty(prhs[2],0,"container"))
//   };
//   assert( (strcmp(container[0],container[1])==0) );
//   bool iscomplex[2]; 
//   logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[0],0,"complex"));
//   iscomplex[0]=*iscomp;
//   iscomp=(logical *)mxGetPr(mxGetProperty(prhs[1],0,"complex"));
//   iscomplex[1]=*iscomp;
//   assert( iscomplex[0] == iscomplex[1] );
//   
//   if (strcmp(container[0], "host")==0){
//     if (iscomplex[0])
//       mx_mThrust_eqVec<thrust::complex<T>,thrust::host_vector<thrust::complex<T> >, thrust::host_vector<logical> >(prhs);
//     else
//       mx_mThrust_eqVec<T ,thrust::host_vector<T>, thrust::host_vector<logical> >(prhs);
//   }else if (strcmp(container[0], "device")==0){
//     if (iscomplex[0])
//       mx_mThrust_eqVec<thrust::complex<T>,thrust::device_vector<thrust::complex<T> >, thrust::device_vector<logical> >(prhs);
//     else
//       mx_mThrust_eqVec<T ,thrust::device_vector<T>, thrust::device_vector<logical> >(prhs);
//   }else{
//     mxFree(container[0]);mxFree(container[1]);mxFree(container[2]);
//     mexErrMsgIdAndTxt("mThrustToolbox:eq","Unknow container."); 
//   }
//   mxFree(container[0]);mxFree(container[1]);mxFree(container[2]);
// }
// 
// template  <typename T>
// void mx_mThrust_eqCst_type( const mxArray *prhs[] ){
//   char *container[] = {
//     mxArrayToString(mxGetProperty(prhs[0],0,"container")),
//     mxArrayToString(mxGetProperty(prhs[2],0,"container"))
//   };
//   bool iscomplex[2]; 
//   logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[0],0,"complex"));
//   iscomplex[0]=*iscomp;
//   iscomplex[1]=mxIsComplex(prhs[1]);
//   assert( iscomplex[0] == iscomplex[1] );
//   
//   if (strcmp(container[0], "host")==0){
//     if (iscomplex[0])
//       mx_mThrust_eqCst<thrust::complex<T>,thrust::host_vector<thrust::complex<T> >, thrust::host_vector<logical> >(prhs);
//     else
//       mx_mThrust_eqCst<T ,thrust::host_vector<T>, thrust::host_vector<logical> >(prhs);
//   }else if (strcmp(container[0], "device")==0){
//     if (iscomplex[0])
//       mx_mThrust_eqCst<thrust::complex<T>,thrust::device_vector<thrust::complex<T> >, thrust::device_vector<logical> >(prhs);
//     else
//       mx_mThrust_eqCst<T ,thrust::device_vector<T>, thrust::device_vector<logical> >(prhs);
//   }else{
//     mxFree(container[0]);mxFree(container[1]);
//     mexErrMsgIdAndTxt("mThrustToolbox:eq","Unknow container."); 
//   }
//   mxFree(container[0]);mxFree(container[1]);
// }
// 
// void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
// {
//   MEXPRINTF("starting");
//   if (nlhs !=0 || nrhs != 3) 
//     mexErrMsgTxt("Bad input.");
//   assert( mxIsClass(prhs[0],"mThrust") && mxIsClass(prhs[2],"mThrust") );
//   logical isScalar=false;
//   logical ismThrust=mxIsClass(prhs[1],"mThrust");
//   if (!ismThrust) isScalar=mxIsScalar(prhs[1]);
//   assert( ismThrust || isScalar );
//   
//   char *type =mxArrayToString(mxGetProperty(prhs[0],0,"type"));
//   if (strcmp(type, "double")==0){
//     if (isScalar) mx_mThrust_eqCst_type<double>(prhs);else mx_mThrust_eqVec_type<double>(prhs);
//   }else if (strcmp(type, "int")==0){
//     if (isScalar) mx_mThrust_eqCst_type<int>(prhs);else mx_mThrust_eqVec_type<int>(prhs);
//   }else if (strcmp(type, "float")==0){
//     if (isScalar) mx_mThrust_eqCst_type<float>(prhs);else mx_mThrust_eqVec_type<float>(prhs);
//   }else if (strcmp(type, "unsigned")==0){
//     if (isScalar) mx_mThrust_eqCst_type<unsigned>(prhs);else mx_mThrust_eqVec_type<unsigned>(prhs);
//   }else{
//     mxFree(type);
//     mexErrMsgIdAndTxt("mThrustToolbox:eq","type not implemented.");
//   }
//   mxFree(type);
//   MEXPRINTF("ending");
// }

#include "mex_mThrust_cmathop_gen.cu"