/*! \file mex_mThrust_set.cu
 *  \brief MEX-file : delete a C++ \p gTArray object (GPU).
 * 
 *  The gateway Cuda/C++ function \p mex_gTArrayDestroy destroy a C++ \p gTArray object 
 *  pointed by a handle.
 * 
 *  &copy;Matlab syntax <tt><B>mex_gTArrayDelete(h,type)</B></tt> where
 *    - \p h  handle representing a C++ \p gTArray object.
 *    - \p type of \p gTArray object given by \p h. Must be 'int', 'unsigned', 'float' or 'double'.
 * 
 *  The following &copy;Matlab code snippet demonstrates how to use this gateway function \p mex_gTArrayDelete 
 *  \code
 *  >> hA=mex_gTArrayCreate(int32(500),'double');
 *  ...
 *  >> mex_gTArrayDelete(hA,'double');
 *  \endcode
 * 
 * \sa  mex_gTArrayCreate
 */
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"
#include <time.h>


// #define PRINT_DEBUG(C,T,N,A) printf("%sdebug : %s_vector<%s>[%d] (memory adress: %#lX)%s\n",MEXCOLOR,C,T,N,(MEMORY_TYPE)A,TEXTCOLOR_END);

#include <thrust/random.h>
#include <thrust/host_vector.h>
#include <thrust/device_vector.h>
#include <thrust/transform.h>
#include <thrust/iterator/counting_iterator.h>

__host__ __device__
unsigned int hash(unsigned int a)
{
    a = (a+0x7ed55d16) + (a<<12);
    a = (a^0xc761c23c) ^ (a>>19);
    a = (a+0x165667b1) + (a<<5);
    a = (a+0xd3a2646c) ^ (a<<9);
    a = (a+0xfd7046c5) + (a<<3);
    a = (a^0xb55a4f09) ^ (a>>16);
    return a;
}

template <typename T>
struct prgreal
{
    T a, b;
    unsigned int t;

    __host__ __device__
    prgreal(T _a=0.f, T _b=1.f, unsigned int _t=0 ) : a(_a), b(_b), t(_t) {};

    __host__ __device__
        T operator()(unsigned int thread_id) const
        {
	  unsigned int seed = hash(thread_id+t);
          //thrust::default_random_engine rng;
	  thrust::minstd_rand rng(seed);
          thrust::uniform_real_distribution<T> dist(a, b);
          //  rng.discard(n);

          return dist(rng);
        }
};

template <typename T>
struct zprgreal
{
  T a, b;
  unsigned int t;
  
  __host__ __device__
  zprgreal(T _a=0.f, T _b=1.f, unsigned int _t=0 ) : a(_a), b(_b), t(_t) {};
  
  __host__ __device__
  thrust::complex<T> operator()(unsigned int thread_id) const
  {
    unsigned int seed = hash(thread_id+t);
    //thrust::default_random_engine rng;
    thrust::minstd_rand rng(seed);
    thrust::uniform_real_distribution<T> dist(a, b);
    //  rng.discard(n);
    
    return thrust::complex<T>(dist(rng),dist(rng));
  }
};

struct prgbool
{
    unsigned int t;

    __host__ __device__
    prgbool(unsigned int _t=0 ) : t(_t) {};

    __host__ __device__
        bool operator()(unsigned int thread_id) const
        {
	  unsigned int seed = hash(thread_id+t);
          //thrust::default_random_engine rng;
	  thrust::minstd_rand rng(seed);
          thrust::uniform_real_distribution<float> dist(0.f, 1.f);
          //  rng.discard(n);

          return (dist(rng)<0.5f);
        }
};

struct zprgbool
{
  unsigned int t;
  
  __host__ __device__
  zprgbool(unsigned int _t=0 ) : t(_t) {};
  
  __host__ __device__
  thrust::complex<bool> operator()(unsigned int thread_id) const
  {
    unsigned int seed = hash(thread_id+t);
    //thrust::default_random_engine rng;
    thrust::minstd_rand rng(seed);
    thrust::uniform_real_distribution<float> dist(0.f, 1.f);
    //  rng.discard(n);
    
    return thrust::complex<bool>(dist(rng)<0.5f,dist(rng)<0.5f);
  }
};

template <typename T>
struct prgint
{
    T a, b;
    unsigned int t;

    __host__ __device__
    prgint(T _a=0, T _b=10, unsigned int _t=0 ) : a(_a), b(_b), t(_t) {};

    __host__ __device__
        T operator()(unsigned int thread_id) const
        {
	  unsigned int seed = hash(thread_id+t);
          //thrust::default_random_engine rng;
	  thrust::minstd_rand rng(seed);
          thrust::uniform_int_distribution<T> dist(a, b);
          //  rng.discard(n);

          return dist(rng);
        }
};

template <typename T>
struct zprgint
{
  T a, b;
  unsigned int t;
  
  __host__ __device__
  zprgint(T _a=0, T _b=10, unsigned int _t=0 ) : a(_a), b(_b), t(_t) {};
  
  __host__ __device__
  thrust::complex<T> operator()(unsigned int thread_id) const
  {
    unsigned int seed = hash(thread_id+t);
    //thrust::default_random_engine rng;
    thrust::minstd_rand rng(seed);
    thrust::uniform_int_distribution<T> dist(a, b);
    //  rng.discard(n);
    
    return thrust::complex<T>(dist(rng),dist(rng));
  }
};

template  <typename T>
void mx_mThrust_rand_type_int(const mxArray *prhs[],
                          char *container,logical iscomplex,T a,T b, unsigned int seed){
  if (strcmp(container, "host")==0){
    if (iscomplex){
      thrust::host_vector<thrust::complex<T> > &pV=get_object< thrust::host_vector<thrust::complex<T> > >(prhs[0]);
      thrust::transform(thrust::counting_iterator<unsigned int>(0),
                        thrust::counting_iterator<unsigned int>(pV.size()),
                        pV.begin(),
                        zprgint<T>(a,b,seed));
    }else{
      thrust::host_vector<T> &pV=get_object< thrust::host_vector<T> >(prhs[0]);
      thrust::transform(thrust::counting_iterator<unsigned int>(0),
                      thrust::counting_iterator<unsigned int>(pV.size()),
                      pV.begin(),
                      prgint<T>(a,b,seed));
    }
  }else if (strcmp(container, "device")==0){
    if (iscomplex){
      thrust::device_vector<thrust::complex<T> > &pV=get_object< thrust::device_vector<thrust::complex<T> > >(mxGetProperty(prhs[0],0,"handle"));
      thrust::transform(thrust::counting_iterator<unsigned int>(0),
                        thrust::counting_iterator<unsigned int>(pV.size()),
                        pV.begin(),
                        zprgint<T>(a,b,seed));
    }else{
      thrust::device_vector<T> &pV=get_object< thrust::device_vector<T> >(mxGetProperty(prhs[0],0,"handle"));
      thrust::transform(thrust::counting_iterator<unsigned int>(0),
                        thrust::counting_iterator<unsigned int>(pV.size()),
                        pV.begin(),
                        prgint<T>(a,b,seed));
    }
    
  }
}

template  <typename T>
void mx_mThrust_rand_type_real(const mxArray *prhs[],
                              char *container,logical iscomplex,T a,T b, unsigned int seed){
  if (strcmp(container, "host")==0){
    if (iscomplex){
      thrust::host_vector<thrust::complex<T> > &pV=get_object< thrust::host_vector<thrust::complex<T> > >(mxGetProperty(prhs[0],0,"handle"));
      thrust::transform(thrust::counting_iterator<unsigned int>(0),
                        thrust::counting_iterator<unsigned int>(pV.size()),
                        pV.begin(),
                        zprgreal<T>(a,b,seed));
    }else{
      thrust::host_vector<T> &pV=get_object< thrust::host_vector<T> >(mxGetProperty(prhs[0],0,"handle"));
      thrust::transform(thrust::counting_iterator<unsigned int>(0),
                        thrust::counting_iterator<unsigned int>(pV.size()),
                        pV.begin(),
                        prgreal<T>(a,b,seed));
    }
  }else if (strcmp(container, "device")==0){
    if (iscomplex){
      thrust::device_vector<thrust::complex<T> > &pV=get_object< thrust::device_vector<thrust::complex<T> > >(mxGetProperty(prhs[0],0,"handle"));
      thrust::transform(thrust::counting_iterator<unsigned int>(0),
                        thrust::counting_iterator<unsigned int>(pV.size()),
                        pV.begin(),
                        zprgreal<T>(a,b,seed));
    }else{
      thrust::device_vector<T> &pV=get_object< thrust::device_vector<T> >(mxGetProperty(prhs[0],0,"handle"));
      thrust::transform(thrust::counting_iterator<unsigned int>(0),
                        thrust::counting_iterator<unsigned int>(pV.size()),
                        pV.begin(),
                        prgreal<T>(a,b,seed));
    }
    
  }
}

// prhs[0] -> input/output mThrust object
// prhs[1] -> a
// prhs[2] -> b 
// prhs[3] -> seed (int32)
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 4) 
    mexErrMsgTxt("Bad input.");
  assert(mxIsClass(prhs[0],"mThrust"));
  assert(mxIsClass(prhs[3],"uint32"));
  char *type =mxArrayToString(mxGetProperty(prhs[0],0,"type"));
  char *container =mxArrayToString(mxGetProperty(prhs[0],0,"container"));
  logical *iscomplex=(logical *) mxGetPr(mxGetProperty(prhs[0],0,"complex"));
  
  unsigned *seed = (unsigned *) mxGetPr(prhs[3]);
  
  if( mxGetNumberOfElements(prhs[1])!=1 )
    mexErrMsgIdAndTxt("mThrust:rand","2nd input value must be a scalar.");
  if( mxGetNumberOfElements(prhs[2])!=1 )
    mexErrMsgIdAndTxt("mThrust:rand","3th input value must be a scalar.");
  
  if (strcmp(type, "double")==0){
    double *pa = (double *) mxGetPr(prhs[1]);
    double *pb = (double *) mxGetPr(prhs[2]);
    mx_mThrust_rand_type_real<double>(prhs,container,*iscomplex,*pa,*pb,*seed);
  }else  if (strcmp(type, "int")==0){
    int *pa = (int *) mxGetPr(prhs[1]);
    int *pb = (int *) mxGetPr(prhs[2]);
    mx_mThrust_rand_type_int<int>(prhs,container,*iscomplex,*pa,*pb,*seed);
  }else if (strcmp(type, "unsigned")==0){
    unsigned *pa = (unsigned *) mxGetPr(prhs[1]);
    unsigned *pb = (unsigned *) mxGetPr(prhs[2]);
    mx_mThrust_rand_type_int<unsigned>(prhs,container,*iscomplex,*pa,*pb,*seed);
  }else if (strcmp(type, "float")==0){
    float *pa = (float *) mxGetPr(prhs[1]);
    float *pb = (float *) mxGetPr(prhs[2]);
    mx_mThrust_rand_type_real<float>(prhs,container,*iscomplex,*pa,*pb,*seed);
  }else
    mexErrMsgIdAndTxt("mThrustToolbox:rand","type not implemented.");
}