/*! \file mex_mThrust_plus.cu
 *  \brief MEX-file : delete a C++ \p gTArray object (GPU).
 * 
 *  The gateway Cuda/C++ function \p mex_gTArrayDestroy destroy a C++ \p gTArray object 
 *  pointed by a handle.
 * 
 *  &copy;Matlab syntax <tt><B>mex_gTArrayDelete(h,type)</B></tt> where
 *    - \p h  handle representing a C++ \p gTArray object.
 *    - \p type of \p gTArray object given by \p h. Must be 'int', 'unsigned', 'float' or 'double'.
 * 
 *  The following &copy;Matlab code snippet demonstrates how to use this gateway function \p mex_gTArrayDelete 
 *  \code
 *  >> hA=mex_gTArrayCreate(int32(500),'double');
 *  ...
 *  >> mex_gTArrayDelete(hA,'double');
 *  \endcode
 * 
 * \sa  mex_gTArrayCreate
 */
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"

template <typename T1, typename T2, typename Tout> struct op_functor : public thrust::binary_function<T1,T2,Tout >
{
  __host__ __device__
  Tout operator()(const T1 & z1, const T2& z2) const { 
    return Tout(z1/z2);
  }
};

#define MESS "mThrustToolbox:rdivide"

template <typename Vector1,typename Vector2,typename VectorOut> 
void mx_mThrust_op(const mxArray *prhs[]){
  typedef typename Vector1::value_type T1;
  typedef typename Vector2::value_type T2;
  typedef typename VectorOut::value_type Tout;
  Vector1 &gx = get_object< Vector1 >(mxGetProperty(prhs[0],0,"handle"));
  Vector2 &gy = get_object< Vector2 >(mxGetProperty(prhs[1],0,"handle"));
  VectorOut &gz = get_object< VectorOut >(mxGetProperty(prhs[2],0,"handle"));
  assert((gx.size()==gy.size()) && (gx.size()==gz.size()));
  thrust::transform(gx.begin(),gx.end(), gy.begin(),gz.begin(), op_functor<T1,T2,Tout>());
}


template  <typename T>
void mx_mThrust_op_type(const mxArray *prhs[]){
  char *container[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"container")),
    mxArrayToString(mxGetProperty(prhs[1],0,"container")), 
    mxArrayToString(mxGetProperty(prhs[2],0,"container"))
  };
  assert( (strcmp(container[0],container[1])==0) && (strcmp(container[0],container[2])==0) );
  bool iscomplex[3];
  for (int i=0;i<3;i++){
    logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[i],0,"complex"));
    iscomplex[i]=*iscomp;
  }
  
  if (strcmp(container[0], "host")==0){
    typedef thrust::host_vector<thrust::complex<T> > VecComp;
    typedef thrust::host_vector<T> VecReal;
    
    if (iscomplex[0] && iscomplex[1])
      mx_mThrust_op<VecComp,VecComp,VecComp>(prhs);
    else if (iscomplex[0])
      mx_mThrust_op<VecComp,VecReal,VecComp>(prhs);
    else if (iscomplex[1])
      mx_mThrust_op<VecReal,VecComp,VecComp>(prhs);
    else
      mx_mThrust_op<VecReal,VecReal,VecReal>(prhs);  
  }else if (strcmp(container[0], "device")==0){
    typedef thrust::device_vector<thrust::complex<T> > VecComp;
    typedef thrust::device_vector<T> VecReal;
    
    if (iscomplex[0] && iscomplex[1])
      mx_mThrust_op<VecComp,VecComp,VecComp>(prhs);
    else if (iscomplex[0])
      mx_mThrust_op<VecComp,VecReal,VecComp>(prhs);
    else if (iscomplex[1])
      mx_mThrust_op<VecReal,VecComp,VecComp>(prhs);
    else
      mx_mThrust_op<VecReal,VecReal,VecReal>(prhs);  
  }else{
    mxFree(container[0]);mxFree(container[1]);mxFree(container[2]);
    mexErrMsgIdAndTxt(MESS,"Unknow container."); 
  }
  mxFree(container[0]);mxFree(container[1]);mxFree(container[2]);
}



void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{ 
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 3) 
    mexErrMsgTxt("Bad input.");
  assert(mxIsClass(prhs[0],"mThrust"));
  assert(mxIsClass(prhs[1],"mThrust"));
  assert(mxIsClass(prhs[2],"mThrust"));
  
  char *type[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"type")),
    mxArrayToString(mxGetProperty(prhs[1],0,"type")), 
    mxArrayToString(mxGetProperty(prhs[2],0,"type"))
  };
  assert( (strcmp(type[0],type[1])==0) && (strcmp(type[0],type[2])==0) );
  
  if (strcmp(type[0], "float")==0){
    mx_mThrust_op_type<float>(prhs);
  }else if (strcmp(type[0], "double")==0){
    mx_mThrust_op_type<double>(prhs);
  }else{
    mxFree(type[0]);mxFree(type[1]);mxFree(type[2]);
    mexErrMsgIdAndTxt(MESS,"type not implemented.");
  }
  mxFree(type[0]);mxFree(type[1]);mxFree(type[2]);
  MEXPRINTF("ending");
}