/*! \file mex_mThrust_plus.cu
 *  \brief MEX-file : delete a C++ \p gTArray object (GPU).
 * 
 *  The gateway Cuda/C++ function \p mex_gTArrayDestroy destroy a C++ \p gTArray object 
 *  pointed by a handle.
 * 
 *  &copy;Matlab syntax <tt><B>mex_gTArrayDelete(h,type)</B></tt> where
 *    - \p h  handle representing a C++ \p gTArray object.
 *    - \p type of \p gTArray object given by \p h. Must be 'int', 'unsigned', 'float' or 'double'.
 * 
 *  The following &copy;Matlab code snippet demonstrates how to use this gateway function \p mex_gTArrayDelete 
 *  \code
 *  >> hA=mex_gTArrayCreate(int32(500),'double');
 *  ...
 *  >> mex_gTArrayDelete(hA,'double');
 *  \endcode
 * 
 * \sa  mex_gTArrayCreate
 */
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"

template <typename T> struct functor_xtimesbool
{
   __host__ __device__
  T operator()(const T& x, const logical& y) const
  { 
    return (y ? x :(T)0);
  }
};

template <typename Vector, typename VectorBool> 
void mx_mThrust_timeslogical(const mxArray *prhs[]){
  typedef typename Vector::value_type T;
  
  Vector &gx = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  VectorBool &gy = get_object< VectorBool >(mxGetProperty(prhs[1],0,"handle"));
  Vector &gz = get_object< Vector >(mxGetProperty(prhs[2],0,"handle"));
  assert((gx.size()==gy.size()) && (gx.size()==gz.size()));
  thrust::transform(gx.begin(),gx.end(), gy.begin(),gz.begin(), functor_xtimesbool<T>());
}

template  <typename T>
void mx_mThrust_timeslogical_type(const mxArray *prhs[]){
  char *container[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"container")),
    mxArrayToString(mxGetProperty(prhs[1],0,"container")), 
    mxArrayToString(mxGetProperty(prhs[2],0,"container"))
  };
  assert( (strcmp(container[0],container[1])==0) && (strcmp(container[0],container[2])==0) );
  bool iscomplex[2];
  logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[0],0,"complex"));
  iscomplex[0]=*iscomp;
  iscomp=(logical *)mxGetPr(mxGetProperty(prhs[2],0,"complex"));
  iscomplex[1]=*iscomp;
  assert( iscomplex[0]==iscomplex[1] );
  
  if (strcmp(container[0], "host")==0){
    typedef thrust::host_vector<thrust::complex<T> > VecCplx;
    typedef thrust::host_vector<T >                  VecReal;
    typedef thrust::host_vector<logical>             VecBool;
    if (iscomplex[0])
      mx_mThrust_timeslogical< VecCplx , VecBool>(prhs);
    else
      mx_mThrust_timeslogical< VecReal , VecBool>(prhs);
  }else if (strcmp(container[0], "device")==0){
    typedef thrust::device_vector<thrust::complex<T> > VecCplx;
    typedef thrust::device_vector<T >                  VecReal;
    typedef thrust::device_vector<logical>             VecBool;
    if (iscomplex[0])
      mx_mThrust_timeslogical< VecCplx , VecBool>(prhs);
    else
      mx_mThrust_timeslogical< VecReal , VecBool>(prhs);
  }else{
    mxFree(container[0]);mxFree(container[1]);mxFree(container[2]);
    mexErrMsgIdAndTxt("mThrustToolbox:plus","Unknow container."); 
  }
  mxFree(container[0]);mxFree(container[1]);mxFree(container[2]);
}

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 3) 
    mexErrMsgTxt("Bad input.");
  assert(mxIsClass(prhs[0],"mThrust"));
  assert(mxIsClass(prhs[1],"mThrust"));
  assert(mxIsClass(prhs[2],"mThrust"));
  
  char *type[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"type")),
    mxArrayToString(mxGetProperty(prhs[1],0,"type")), 
    mxArrayToString(mxGetProperty(prhs[2],0,"type"))
  };
  assert( strcmp(type[0],type[2])==0 );
  
  if (strcmp(type[0], "int")==0){
    mx_mThrust_timeslogical_type<int>(prhs);
  }else if (strcmp(type[0], "unsigned")==0){
    mx_mThrust_timeslogical_type<unsigned>(prhs);
  }else if (strcmp(type[0], "float")==0){
    mx_mThrust_timeslogical_type<float>(prhs);
  }else if (strcmp(type[0], "double")==0){
    mx_mThrust_timeslogical_type<double>(prhs);
  }else  if (strcmp(type[0], "logical")==0){
    mxFree(type[0]);mxFree(type[1]);mxFree(type[2]);
    mexErrMsgIdAndTxt("mThrustToolbox:timeslogical","operator <times> not implemented for type <logical>.");
  }else{
    mxFree(type[0]);mxFree(type[1]);mxFree(type[2]);
    mexErrMsgIdAndTxt("mThrustToolbox:timeslogical","type not implemented.");
  }
  mxFree(type[0]);mxFree(type[1]);mxFree(type[2]);
  MEXPRINTF("ending");
//   if (strcmp(type, "int")==0){
//     if (strcmp(container, "host")==0){
//       mx_mThrust_func<int,thrust::host_vector<int>, thrust::host_vector<logical> >(prhs[0],prhs[1],prhs[2]);
//     }else if (strcmp(container, "device")==0){
//       mx_mThrust_func<int,thrust::device_vector<int>, thrust::device_vector<logical> >(prhs[0],prhs[1],prhs[2]);
//     }else
//       mexErrMsgIdAndTxt("mThrustToolbox:times","Unknow container.");
//   }else if (strcmp(type, "unsigned")==0){
//     if (strcmp(container, "host")==0){
//       mx_mThrust_func<unsigned,thrust::host_vector<unsigned>, thrust::host_vector<logical> >(prhs[0],prhs[1],prhs[2]);
//     }else if (strcmp(container, "device")==0){
//       mx_mThrust_func<unsigned,thrust::device_vector<unsigned>, thrust::device_vector<logical> >(prhs[0],prhs[1],prhs[2]);
//     }else
//       mexErrMsgIdAndTxt("mThrustToolbox:times","Unknow container.");
//   }else if (strcmp(type, "float")==0){
//     if (strcmp(container, "host")==0){
//       mx_mThrust_func<float,thrust::host_vector<float>, thrust::host_vector<logical> >(prhs[0],prhs[1],prhs[2]);
//     }else if (strcmp(container, "device")==0){
//       mx_mThrust_func<float,thrust::device_vector<float>, thrust::device_vector<logical> >(prhs[0],prhs[1],prhs[2]);
//     }else
//       mexErrMsgIdAndTxt("mThrustToolbox:times","Unknow container.");
//   }else if (strcmp(type, "double")==0){
//     if (strcmp(container, "host")==0){
//       mx_mThrust_func<double,thrust::host_vector<double>, thrust::host_vector<logical> >(prhs[0],prhs[1],prhs[2]);
//     }else if (strcmp(container, "device")==0){
//       mx_mThrust_func<double,thrust::device_vector<double>, thrust::device_vector<logical> >(prhs[0],prhs[1],prhs[2]);
//     }else
//       mexErrMsgIdAndTxt("mThrustToolbox:times","Unknow container.");
//   }else  if (strcmp(type, "logical")==0){
//       mexErrMsgIdAndTxt("mThrustToolbox:times","operator <times> not implemented for type <logical>.");
//   }else
//     mexErrMsgIdAndTxt("mThrustToolbox:times","type not implemented.");
}